<?php

namespace Controller;

use Zeedhi\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use \Util\Exception;

class Payment extends \Zeedhi\Framework\Controller\Simple {

	protected $util;
	protected $paymentService;
	protected $registerService;
	protected $vendaAPI;
	protected $tableService;
	protected $consumerService;
	protected $utilAPI;
	protected $caixaAPI;
	protected $deliveryService;
	protected $WiBXApi;

	const IDORIGEMVENDA_TABLE = 'MES_PKR';
	const IDORIGEMVENDA_BILL = 'CMD_PKC';
	const IDORIGEMVENDA_REGISTER = 'BAL_MOB';
    const IDORIGEMVENDA_DRIVETHU = 'CMD_THR';

	public function __construct(
		\Util\Util $util,
		\Service\Payment $paymentService,
		\Service\Register $registerService,
		\Odhen\API\Service\Venda $vendaAPI,
		\Service\Table $tableService,
		\Odhen\API\Service\Consumidor $consumerService,
		\Odhen\API\Util\Util $utilAPI,
		\Odhen\API\Service\Caixa $caixaAPI,
		\Service\Delivery $deliveryService,
		\Odhen\API\Service\WiBX $WiBXApi
	){
		$this->util = $util;
		$this->paymentService = $paymentService;
		$this->registerService = $registerService;
		$this->vendaAPI = $vendaAPI;
		$this->tableService = $tableService;
		$this->consumerService = $consumerService;
		$this->utilAPI = $utilAPI;
		$this->caixaAPI = $caixaAPI;
		$this->deliveryService = $deliveryService;
		$this->WiBXService = $WiBXApi;
	}

	private function getIDORIGEMVENDA($IDMODULO, $IDUTCXDRIVETHU, $NRVENDAREST = null) {
		$IDORIGEMVENDA = null;
		switch ($IDMODULO) {
			case 'M':
				$IDORIGEMVENDA = self::IDORIGEMVENDA_TABLE;
				break;
			case 'C':
				$IDORIGEMVENDA = $IDUTCXDRIVETHU === 'S' ? self::IDORIGEMVENDA_DRIVETHU : self::IDORIGEMVENDA_BILL;
				break;
			case 'D':
				$IDORIGEMVENDA = $this->paymentService->getIDORIGEMVENDADLV($NRVENDAREST);
				break;
			default:
				$IDORIGEMVENDA = self::IDORIGEMVENDA_REGISTER;
				break;
		}
		return $IDORIGEMVENDA;
	}

	private function fixDiscount($porcentagemDesconto, $totalSemDesconto) {
		return $totalSemDesconto * $porcentagemDesconto;
	}

    public function preparePayAccount(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->util->getParams($params);

        $chave   = !empty($params['chave']) ? $params['chave'] : null;
        $session = $this->util->getSessionVars($chave);
        if (isset($params['IDMODULO'])){
            $session['IDMODULO'] = $params['IDMODULO'];
        }

        if ($params['DELIVERY']){
            // prepara parametros para vendas originadas no modo delivery
            self::prepareDeliveryDataSale($params);
        } else {
            //Trata as bandeiras dos recebimentos realizados pelas integrações Cielo LIO, GetNet e PagSeguro.
            foreach ($params['TIPORECE'] as $indice => $paramUnit) {
                if ($paramUnit['IDTPTEF'] === '7' || $paramUnit['IDTPTEF'] === '8' || $paramUnit['IDTPTEF'] === '9' || $paramUnit['IDTPTEF'] === '10' || $paramUnit['IDTPTEF'] === '4') {
					// log:
					$this->util->LogBandeira("Bandeira em tratamento -> bandeira inicial: {$params['TIPORECE'][$indice]['CDBANCARTCR']}");
					$params['TIPORECE'][$indice]['CDBANCARTCR'] = self::trataBandeirasIntegracoes($paramUnit['CDBANCARTCR'], $paramUnit['IDTIPORECE']);
					// log:
					$this->util->LogBandeira("Fim de tratamento -> bandeira de retorno: {$params['TIPORECE'][$indice]['CDBANCARTCR']}");
				}
            }
        }
        
        $DATASALE = $params['DATASALE'];

		$params['DATASALE']['PromocoesIntUtilizadas']=array();
		$params['descontoPromocoesIntOffers'] = (isset($params['descontoPromocoesIntOffers']))?$params['descontoPromocoesIntOffers']:null;
		if(!empty($params['descontoPromocoesIntOffers'])){
			array_push( $params['DATASALE']['PromocoesIntUtilizadas'], $params['descontoPromocoesIntOffers'] );
		}
		$params['descontoPromocoesIntPayment'] = (isset($params['descontoPromocoesIntPayment']))?$params['descontoPromocoesIntPayment']:null;
		if(!empty($params['descontoPromocoesIntPayment'])){
			array_push( $params['DATASALE']['PromocoesIntUtilizadas'], $params['descontoPromocoesIntPayment'] );
		}
		$params['descontoPromocoesIntWallet'] = (isset($params['descontoPromocoesIntWallet']))?$params['descontoPromocoesIntWallet']:null;
		if(!empty($params['descontoPromocoesIntWallet'])){
			array_push( $params['DATASALE']['PromocoesIntUtilizadas'], $params['descontoPromocoesIntWallet'] );
		}
	
		$params['DATASALE']['NRCPFFIDUDS'] = (isset($params['NRCPFFIDUDS'])?$params['NRCPFFIDUDS']:isset($DATASALE['CPFUDS']))?$DATASALE['CPFUDS']:null;
		$params['DATASALE']['VRDESCCASHUDS'] = (isset($params['VRDESCCASHUDS']))?$params['VRDESCCASHUDS']:null;

        // Parâmetros utilizados no pagamento.
		$paymentParams = array(
			'TIPORECE' 		  => $params['TIPORECE'],
			'ITEMVENDA' 	  => $params['ITEMVENDA'],
			'DATASALE' 		  => $params['DATASALE'],
			'CDCLIENTE' 	  => $params['CDCLIENTE'],
			'CDCONSUMIDOR' 	  => $params['CDCONSUMIDOR'],
			'NRMESA' 		  => $params['NRMESA'],
			'NRPESMESAVEN' 	  => $params['NRPESMESAVEN'],
			'NRVENDAREST' 	  => !empty($params['NRVENDAREST']) ? $params['NRVENDAREST'] : null,
			'NRCOMANDA' 	  => !empty($params['NRCOMANDA']) ? $params['NRCOMANDA'] : null,
			'chave' 		  => $chave,
			'saleCode' 		  => $params['saleCode'] . $chave,
			'arrayPosicoes'   => $arrayPosicoes = self::formatPositions($params['NRLUGARMESA']),
			'TIPODESCONTO' 	  => isset($params['TIPODESCONTO'])? $params['TIPODESCONTO']: null,
			'NRINSCRCONS' 	  => !empty($params['NRINSCRCONS']) ? $params['NRINSCRCONS']: null,
			'DSENDECONSVENDA' => !empty($params['DSENDECONSVENDA']) ? $params['DSENDECONSVENDA']: null,
			'EMAIL' 		  => !empty($params['EMAIL']) ? $params['EMAIL']: null,
			'NOMECONS' 		  => !empty($params['NOMECONS']) ? $params['NOMECONS'] : null,
			'logServico' 	  => !empty($params['servico']['logServico']) ? $params['servico']['logServico'] : null,
			'logDesconto' 	  => !empty($params['desconto']['logDesconto']) ? $params['desconto']['logDesconto'] : null,
			'supervisorServ'  => !empty($params['servico']['CDSUPERVISOR']) ? $params['servico']['CDSUPERVISOR'] : $session['CDOPERADOR'],
			'supervisorDesc'  => !empty($params['desconto']['CDSUPERVISOR']) ? $params['desconto']['CDSUPERVISOR'] : $session['CDOPERADOR'],
			'CDVENDEDOR' 	  => $params['CDVENDEDOR'],
			'session' 		  => $session,
			'motivoDesconto'  => !empty($params['desconto']['motivoDesconto']) ? $params['desconto']['motivoDesconto'] : null,
			'CDGRPOCORDESC'   => !empty($params['desconto']['CDGRPOCORDESC']) ? $params['desconto']['CDGRPOCORDESC'] : null,
			'DSOBSFINVEN' 	  => !empty($params['DSOBSFINVEN']) ? $params['DSOBSFINVEN'] : null,
			'ITVENDADES' 	  => !empty($params['ITVENDADES']) ? $params['ITVENDADES'] : null,
			'DELIVERY'	 	  => $params['DELIVERY'],
			'PRODSENHAPED' 	  => !empty($params['PRODSENHAPED']) ? $params['PRODSENHAPED'] : null,
			'INFORMAEMAIL' 	  => !empty($params['INFORMAEMAIL']) ? $params['INFORMAEMAIL'] : false,
			'DSEMAILCONS' 	  => !empty($params['DSEMAILCONS']) ? $params['DSEMAILCONS'] : null
		);

		if (isset($paymentParams['TIPORECE'][0]['ADIANTAMENTO']) && $paymentParams['TIPORECE'][0]['ADIANTAMENTO']){
            // ADIANTAMENTO
            $result = self::advancePayAccount($paymentParams);
		} else {
            // PAGAMENTO PADRÃO
            $result = self::payAccount($paymentParams);
        }

		if(!$result['error']) {
			$this->paymentService->handleRemovePayment($params, $session);
		}

		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('payAccount', $result));
		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('advancePayAccount', !empty($result) ? $result : array()));
    }

    public function payAccount($allParams){

        set_time_limit(0); // bloqueia timeout na venda

        $result = array(
            'error'   => true,
            'message' => ''
        );

        try {
            // Verifica se a nota para esta venda já foi transmitida.
            // Este procedimento faz parte de uma rotina que evita o envio da nota novamente após a mesma já ter sido transmitida.
            // Caso o erro ocorra após a transmissão da nota, não podemos realizar a transmissão novamente.
            try {
                if ($allParams['session']['IDTPEMISSAOFOS'] == 'SAT'){
                    // NOTAS SAT
                    $year = date('Y');
                    $month = date('m');
                    $day = date('d');

                    $XMLPath = realpath('../../../') . DIRECTORY_SEPARATOR . 'SAT' . DIRECTORY_SEPARATOR . $allParams['session']['CDFILIAL'] . DIRECTORY_SEPARATOR . $allParams['session']['CDCAIXA'] . DIRECTORY_SEPARATOR . $year . '-' . $month . '-' . $day . DIRECTORY_SEPARATOR . 'LOGS' . DIRECTORY_SEPARATOR;
                    $vendaTransmitida = file_exists($XMLPath . $allParams['saleCode'] . ".xml");
                    $storedXMLPath = $XMLPath;
                } else if($allParams['session']['IDTPEMISSAOFOS'] == 'FNC'){
                    // NOTAS NFCE
                    $nfceData = $this->caixaAPI->getNFCEData($allParams['session']['CDFILIAL']);

                    $DTEMISSAO = new \DateTime();
                    $typePath = $nfceData['IDAMBTRABNFCE'] == '2' ? 'homologacao' : 'producao';
                    $XMLPath = realpath('../../../') . DIRECTORY_SEPARATOR . 'NFCE' . DIRECTORY_SEPARATOR   . 'NOTAS' . DIRECTORY_SEPARATOR . $typePath . DIRECTORY_SEPARATOR;
                    $nfceMoment = $allParams['session']['NRORG'] . '-' . $allParams['session']['CDFILIAL'] . '-' . $allParams['session']['CDCAIXA'] . '_' . $DTEMISSAO->format('Y-m-d') . DIRECTORY_SEPARATOR;
                    $storedXMLPath = $XMLPath . "ENVIADAS" . DIRECTORY_SEPARATOR . $nfceMoment;

                    $vendaTransmitida = file_exists($storedXMLPath . $allParams['saleCode'] . ".xml");
                }
            } catch (\Exception $e){
                $vendaTransmitida = false;
            }

            // alteração realizada a pedido do madero
			// altera o cdtiporece de acordo com a carteira digital selecionada na integração da sitef
			if ($allParams['TIPORECE'][0]['IDTIPORECE'] === 'H' && isset($allParams['TIPORECE'][0]['digitalWallets'])) {
				$newTiporeceName = self::getDigitalWalletName($allParams['TIPORECE'][0]['digitalWallets'], $allParams['TIPORECE'][0]['selectedDigitalWallet']);

				$recebimentosH = $this->paymentService->getTiporeceH();

				$updatedTiporece = self::checkIfTiporeceIsSaved($newTiporeceName, $recebimentosH);

				if ($updatedTiporece == null){
					$result = Array(
						'error' => true,
						'message' => 'Tipo de recebimento selecionado "' . $newTiporeceName . '" não foi encontrado.'
					);

					return $result;
				}

				$tiporeceBackupValues = array(
					'CDTIPORECE' => $allParams['TIPORECE'][0]['CDTIPORECE'],
					'NMTIPORECE' => $allParams['TIPORECE'][0]['DSBUTTON']
				);

				$allParams['TIPORECE'][0]['CDTIPORECE'] = $updatedTiporece['CDTIPORECE'];
				$allParams['TIPORECE'][0]['DSBUTTON'] = $updatedTiporece['NMTIPORECE'];
			}

            // Verifica o estado do caixa.
            $estadoCaixa = $this->caixaAPI->getEstadoCaixa($allParams['session']['CDFILIAL'], $allParams['session']['CDCAIXA'], $allParams['session']['NRORG']);
            if ($estadoCaixa['estado'] !== 'aberto'){
                throw new \Exception('Não foi possível realizar a venda. O caixa se encontra fechado.');
            }

            /*** EVITA VENDAS DUPLICADAS ***/
			// Caso o backend retorne antes da hora, este processo evita com que o operador retransmita o mesmo pagamento.
			// IDSTORDER = E - erro, C - concluído com sucesso, P - pendente (em processamento)
			$saleCodeObject = $this->paymentService->checkSaleCode($allParams['session']['CDFILIAL'], $allParams['NRVENDAREST'], $allParams['NRCOMANDA'], $allParams['saleCode']);
			if ($saleCodeObject && !$vendaTransmitida){
				if ($saleCodeObject['IDSTORDER'] == 'P'){
					$result['error'] = true;

					$reqDate = substr($saleCodeObject['DTHRINCREQ'], 0, -4);
					$reqDate = date_create($reqDate);
					$minutesDiff = $reqDate->diff(new \DateTime())->format('%I');
					$secondsDiff = $reqDate->diff(new \DateTime())->format('%S');

					if (intval($minutesDiff) >= 1) {
						$this->paymentService->updateSaleCode(
							'E', // E - erro
							json_encode($result),
							$allParams['session']['CDFILIAL'],
							$allParams['NRVENDAREST'],
							$allParams['NRCOMANDA'],
							$allParams['saleCode']
						);

						$result['message'] = 'Tempo para processamento da venda excedido. Favor tentar novamente.';
						$result['resetSaleCode'] = true;
					} else {
						$result['message'] = 'A requisição está em processamento, por favor aguarde um momento e tente novamente. <br><br>
							Tempo decorrido: ' . $minutesDiff . ':' . $secondsDiff . '<br> Tempo máximo: 01:00';
					}
				} else {
					$result = json_decode($saleCodeObject['DSOPERACAO'], true);
					if ($saleCodeObject['IDSTORDER'] == 'E') {
						// última venda com erro; reseta o code (CDORDERWAITER) para uma nova venda
						$result['resetSaleCode'] = true;
					}
				}
			} else {
                if (!$vendaTransmitida){ // Caso a venda já tiver sido transmitida, não grava o código.
                    $this->paymentService->insertSaleCode(
                        'P', // P - pendente (em processamento)
                        json_encode($result),
                        $allParams['session']['CDFILIAL'],
                        $allParams['NRVENDAREST'],
                        $allParams['NRCOMANDA'],
                        $allParams['saleCode']
                    );
                }

                $VRDESCONTO = $allParams['DATASALE']['VRDESCONTO'];

				$allParams['CDFILIAL'] = $allParams['session']['CDFILIAL'];
				$allParams['CDLOJA']   = $allParams['session']['CDLOJA'];
				$allParams['CDCAIXA']  = $allParams['session']['CDCAIXA'];
				$NRORG    = $allParams['session']['NRORG'];
				$CDOPERADOR = $allParams['session']['CDOPERADOR'];

				$allParams['CDCLIENTE']    = $allParams['CDCLIENTE'] ? $allParams['CDCLIENTE'] : null;
				$allParams['CDCONSUMIDOR'] = $allParams['CDCONSUMIDOR'] ? $allParams['CDCONSUMIDOR'] : null;
				if (!$allParams['CDCONSUMIDOR']){
					$NMCONSVEND = 'CONSUMIDOR FINAL';
				} else {
					$NMCONSVEND =  $this->paymentService->findNMCONSUMIDOR($allParams['CDCLIENTE'], $allParams['CDCONSUMIDOR'], $NRORG);
					if (!$allParams['NRINSCRCONS']) {
						$allParams['NRINSCRCONS'] = $this->paymentService->findNRCPFRESPCON($allParams['CDCLIENTE'], $allParams['CDCONSUMIDOR'], $NRORG);
					}
				}
				if ($allParams['NOMECONS'] != null && $allParams['NOMECONS'] != ""){
					$NMCONSVEND = $allParams['NOMECONS'];
				}
				$simulatePrinter = false;
				$simulateSaleValidation = false;

				if ($allParams['session']['IDCOLETOR'] !== 'C'){

					if (isset($allParams['TIPORECE']['ADIANTAMENTO']) && $allParams['TIPORECE']['ADIANTAMENTO']){
                        $this->paymentService->searchAndAddAdvancedPayments($allParams['chave'], $allParams['CDFILIAL'], $allParams['CDCAIXA'], $allParams['TIPORECE'], $allParams['NRVENDAREST'], $allParams['NRCOMANDA']);
                    }

					$moneyCurrency = $this->paymentService->getMoneyCurrency($allParams['session']['FILIALVIGENCIA'], $allParams['session']['NRCONFTELA'], $allParams['session']['DTINIVIGENCIA']);

					// Trata de repique com parametrização para taxa de serviço por produto.
					if (!empty($allParams['DATASALE']['REPIQUE']) && $allParams['session']['IDTRATTAXASERV'] === 'P' && $allParams['session']['CDPRODTAXASERV'] !== null && $allParams['session']['IDCOMISVENDA'] != 'N') {
						if ($allParams['DATASALE']['REPIQUE'] == $allParams['DATASALE']['TROCO']) {
							$allParams['DATASALE']['TROCO'] = 0;
						} else {
							$allParams['DATASALE']['TROCO'] = $allParams['DATASALE']['TROCO'] - $allParams['DATASALE']['REPIQUE'];
						}
						$allParams['DATASALE']['TOTAL'] += $allParams['DATASALE']['REPIQUE'];
					}

                    $CDTIPORECETROCO = $moneyCurrency['CDTIPORECE'];
                    foreach ($allParams['TIPORECE'] as $paymentType){
                        if (array_key_exists('TROCO', $paymentType)){
                            if ($paymentType['TROCO']){
                                $currenyChangeType = $this->paymentService->getChangeType($paymentType['CDTIPORECE']);
                                if (!empty($currenyChangeType)) $CDTIPORECETROCO = $currenyChangeType;
                                break;
                            }
                        }
                    }
					$allParams['DATASALE']['PromocoesIntUtilizadas'] = isset($allParams['DATASALE']['PromocoesIntUtilizadas'])?$allParams['DATASALE']['PromocoesIntUtilizadas']:null;
                    $VRTROCOVEND = array(
                        'CDTIPORECE' => $CDTIPORECETROCO,
                        'VRMOVIVEND' => $allParams['DATASALE']['TROCO']
                    );

					$recebimentoDeMesa = $allParams['session']['IDMODULO'] !== 'B';
					$allParams['CDVENDEDOR'] = Empty($allParams['CDVENDEDOR']) ? $allParams['session']['CDVENDEDOR'] : $allParams['CDVENDEDOR'];

					$openingDate = $this->registerService->getRegisterOpeningDate($allParams['CDFILIAL'], $allParams['CDCAIXA']);
					$openingDate = new \DateTime($openingDate['DTABERCAIX']);
					$DTVENDA = new \DateTime();

					if ($recebimentoDeMesa){
						if ($allParams['DELIVERY']){
							$tableData = $this->deliveryService->getInfoOrder($allParams['CDFILIAL'], $allParams['NRVENDAREST']);
						} else {
							$tableData = $this->tableService->getTablesButNotGrouped($allParams['CDFILIAL'], $allParams['NRVENDAREST'], $allParams['NRCOMANDA'], $allParams['NRMESA'], $NRORG, $allParams['session']['IDMODULO']);
						}
						$VRACRCOMANDA = $this->deliveryService->getTaxaEntrega($allParams['CDFILIAL'], $allParams['NRVENDAREST']);

						$result = $this->vendaAPI->vendaMesa(
							$NRORG,
							$allParams['CDFILIAL'],
							$allParams['CDLOJA'],
							$allParams['CDCAIXA'],
							isset($tableData[0]['CDVENDEDOR']) ? $tableData[0]['CDVENDEDOR'] : $allParams['CDVENDEDOR'],
							$allParams['session']['CDOPERADOR'],
							$openingDate,
							$DTVENDA,
                            $allParams['DATASALE']['TOTAL'] + $allParams['DATASALE']['VRTXSEVENDA'] + $allParams['DATASALE']['VRCOUVERT'],
							$allParams['TIPORECE'],
							$NMCONSVEND,
							$allParams['NRINSCRCONS'],
							$allParams['DSENDECONSVENDA'],
							null, // CDSENHAPED
							$VRTROCOVEND,
							$allParams['EMAIL'], // EMAIL
							$VRDESCONTO + $allParams['DATASALE']['FIDELITYVALUE'],
							$allParams['CDCLIENTE'],
							$allParams['CDCONSUMIDOR'],
							$simulatePrinter,
							$simulateSaleValidation,
							$tableData,
							$allParams['arrayPosicoes'],
							$this->getIDORIGEMVENDA($allParams['session']['IDMODULO'], $allParams['session']['IDUTCXDRIVETHU'], $allParams['NRVENDAREST']),
							$allParams['DATASALE']['VRTXSEVENDA'],
                            $allParams['DATASALE']['FIDELITYDISCOUNT'],
                            $allParams['DATASALE']['cashbackUsed4ALL'],
                            $allParams['DATASALE']['FIDELITYVALUE'],
                        	$allParams['motivoDesconto'],
                        	$allParams['CDGRPOCORDESC'],
                        	$allParams['DSOBSFINVEN'],
                        	$allParams['DELIVERY'],
                        	$VRACRCOMANDA,
                        	isset($allParams['DATASALE']['REPIQUE'])? $allParams['DATASALE']['REPIQUE'] : null,
                            $allParams['saleCode'],
                            $allParams['INFORMAEMAIL'],
                            $allParams['DSEMAILCONS'],
                            $allParams['session']['IDENVCUPOMEMAIL'],
							$allParams['DATASALE']['PromocoesIntUtilizadas']
						);

						if (!$result['error']) {
                            $this->paymentService->updateTaxaServico($allParams['CDFILIAL'], $allParams['CDLOJA'], $allParams['NRVENDAREST'], $allParams['NRCOMANDA']);
                            // TODO adicionar validação aqui
                            try {
                                if( ($allParams['CDCLIENTE'] != null) &&
                                ($allParams['CDCONSUMIDOR'] != null)
                                ) {
                                    $clientAndConsumidorExistente = $this->consumerService
                                    ->getConsumerByClientConsumidor(
                                        $allParams['CDCLIENTE'], $allParams['CDCONSUMIDOR']
                                    );
                                    if (!$clientAndConsumidorExistente) {
                                        $this->vendaAPI->setConsumidorVendaToNull(
                                            $allParams['CDFILIAL'], $allParams['CDCAIXA'],
                                            $result['NRSEQVENDA']
                                        );
                                    }
                                }
                            } catch (Exception $e){
                                Exception::logException($e);
                            }
						}
					} else {
						$CDSENHAPED = !empty($allParams['PRODSENHAPED']) ? $allParams['PRODSENHAPED'] : $this->utilAPI->generateCDSENHAPEDodhenPOS($NRORG, $allParams['CDLOJA'], $allParams['CDFILIAL'], $allParams['session']['IDSENHACUP']);

						$allParams['ITEMVENDA'] = self::handleProducts($allParams['session'], $allParams['ITEMVENDA'], $allParams['CDFILIAL'], $allParams['CDLOJA'], $allParams['CDCLIENTE'], $allParams['CDCONSUMIDOR'], $allParams['CDVENDEDOR']);
						$result = $this->vendaAPI->venda(
							$NRORG,
							$allParams['CDFILIAL'],
							$allParams['CDLOJA'],
							$allParams['CDCAIXA'],
							$allParams['CDVENDEDOR'],
							$allParams['session']['CDOPERADOR'],
							$openingDate,
							$DTVENDA,
							$allParams['DATASALE']['TOTAL'],
							$allParams['ITEMVENDA'],
							$allParams['TIPORECE'],
							$NMCONSVEND,
							$allParams['NRINSCRCONS'],
							$allParams['DSENDECONSVENDA'],
							null, // NRSEQVENDA
							$VRTROCOVEND,
							$allParams['EMAIL'], // EMAIL
							$VRDESCONTO + $allParams['DATASALE']['FIDELITYVALUE'] + $allParams['DATASALE']['VRDESCCASHUDS'],
							$allParams['CDCLIENTE'],
							$allParams['CDCONSUMIDOR'],
							$simulatePrinter,
							$simulateSaleValidation,
							$this->getIDORIGEMVENDA($allParams['session']['IDMODULO'], $allParams['session']['IDUTCXDRIVETHU']),
							$CDSENHAPED,
                            $allParams['NRVENDAREST'],
                            $allParams['NRCOMANDA'],
                            $allParams['DATASALE']['fidelityDetails'],
                            $allParams['DATASALE']['FIDELITYDISCOUNT'],
                            $allParams['DATASALE']['cashbackUsed4ALL'],
                            $allParams['DATASALE']['FIDELITYVALUE'],
                            $allParams['DATASALE']['CODPROMO'],
                        	$allParams['motivoDesconto'],
                        	$allParams['CDGRPOCORDESC'],
                        	$allParams['DSOBSFINVEN'],
                        	$allParams['ITVENDADES'],
                        	isset($allParams['DATASALE']['REPIQUE'])? $allParams['DATASALE']['REPIQUE'] : null,
                            $allParams['saleCode'],
                            $allParams['INFORMAEMAIL'],
                            $allParams['DSEMAILCONS'],
                            $allParams['session']['IDENVCUPOMEMAIL'],
                            $allParams['DATASALE']['NRCPFFIDUDS'],
							$allParams['DATASALE']['VRDESCCASHUDS'],
                            $allParams['DATASALE']['PromocoesIntUtilizadas']
						);
						if (!$result['error']){
							if (!empty($result['paramsImpressora'])) {
								$result['paramsImpressora'] = array($result['paramsImpressora']);
							}
                            // TODO adicionar validação aqui
                            try {
                                if(
                                    ($allParams['CDCLIENTE'] != null) &&
                                    ($allParams['CDCONSUMIDOR'] != null)
                                ) {
                                    $clientAndConsumidorExistente = $this->consumerService
                                    ->getConsumerByClientConsumidor(
                                        $allParams['CDCLIENTE'], $allParams['CDCONSUMIDOR']
                                    );
                                    if (!$clientAndConsumidorExistente) {
                                        $this->vendaAPI->setConsumidorVendaToNull(
                                            $allParams['CDFILIAL'], $allParams['CDCAIXA'],
                                            $result['NRSEQVENDA']
                                        );
                                    }
                                }
                            } catch (Exception $e){
                                Exception::logException($e);
                            }
						}

						if (!$result['error'] && $allParams['session']['IDIMPPEDPROD'] === 'S') {
							$printOrderCupomResult = $this->paymentService->printOrderCupom($allParams['CDFILIAL'], $allParams['CDLOJA'], $allParams['ITEMVENDA'], $allParams['CDVENDEDOR'], $CDSENHAPED, $result['NRSEQVENDA'], $allParams['session']['CDCAIXA'], $allParams['session']['IDIMPPEDPRCIGE']);

							if (isset($printOrderCupomResult['paramsImpressora']) && count($printOrderCupomResult['paramsImpressora']) > 0){
								$result['paramsImpressora'] = array_merge($result['paramsImpressora'], $printOrderCupomResult['paramsImpressora']);
							}

							if (isset($printOrderCupomResult['PedidoSmart']['paramsImpressora']) && count($printOrderCupomResult['PedidoSmart']['paramsImpressora']) > 0){
								$result['impressaoPedidoSmart'] = $printOrderCupomResult['PedidoSmart']['paramsImpressora'];
							}

							if ($printOrderCupomResult['error']) {
								if (!empty($result['mensagemImpressao'])){
									$result['mensagemImpressao'] .= '<br>';
								}
								$result['mensagemImpressao'] .= 'Ocorreu um problema na impressão do cupom de pedido. ' .  $printOrderCupomResult['message'];
							}
						}
					}

					// valida impressão do comprovante TEF
					if (!$result['error'] && !$simulatePrinter && !$simulateSaleValidation){

						$this->paymentService->logServDesc($allParams['logServico'], $allParams['logDesconto'], $allParams['NRMESA'], $allParams['NRCOMANDA'], $allParams['session']['CDFILIAL'], $allParams['session']['CDCAIXA'], $allParams['session']['CDOPERADOR'], $allParams['supervisorServ'], $allParams['supervisorDesc'], $result['NRSEQVENDA']);
						// verifica se há comprovantes a serem imprimidos
						$arrTiporece = array_filter($allParams['TIPORECE'], function($recebimento){
							return !empty($recebimento['STLPRIVIA']);
						});

						if (!empty($arrTiporece)){
							$printTEFVoucherResult = $this->paymentService->printTEFVoucher($arrTiporece, $allParams['CDFILIAL'], $allParams['CDCAIXA'], $NRORG);
							if(isset($printTEFVoucherResult['paramsImpressora'])&&count($printTEFVoucherResult['paramsImpressora'])>0){
								$result['paramsImpressora'] = array_merge($result['paramsImpressora'], $printTEFVoucherResult['paramsImpressora']);
							}
							if ($printTEFVoucherResult['error']){
								if (!empty($result['mensagemImpressao'])){
									$result['mensagemImpressao'] .= '<br>';
								}
								$result['mensagemImpressao'] .= $printTEFVoucherResult['message'];
							} else {
								if (!empty($printTEFVoucherResult['data'])){
                                    $result['dadosImpressao']['TEFVOUCHER'] = array();
                                    foreach ($printTEFVoucherResult['data'] as $tefData){
                                        array_push($result['dadosImpressao']['TEFVOUCHER'], $tefData);
                                    }
								}
							}
						}
					}
				} else {
					$result = $this->paymentService->initiatesAnticipate(
						$allParams['chave'],
						$allParams['DATASALE'],
						$allParams['TIPORECE'],
						$allParams['CDCLIENTE'],
						$allParams['CDCONSUMIDOR'],
						$allParams['NRMESA'],
						$allParams['NRVENDAREST'],
						$allParams['NRCOMANDA'],
						$allParams['NRINSCRCONS'],
						$allParams['arrayPosicoes']
					);
				}

				if (!$result['error']) {
					$resultDadosMesa = $this->tableService->dadosMesa(
						$allParams['CDFILIAL'],
						$allParams['CDLOJA'],
						$allParams['NRCOMANDA'],
						$allParams['NRVENDAREST'],
						false
					);
					$result['IDSTMESAAUX'] = !empty($resultDadosMesa['IDSTMESAAUX']) ? $resultDadosMesa['IDSTMESAAUX'] : 'D';

					if ($allParams['session']['IDCRMBONUS'] == 'S' && $allParams['DATASALE']['PHONECRM'] != NULL) {

						try{

							$telefone = $allParams['DATASALE']['PHONECRM'];
        					$amount = $allParams['DATASALE']['TOTAL'];
        					$URL = $allParams['session']['CDURLCRMBONUS'] . '/api/pos/finaliza-venda-full';
        					$autorizacaoCrm = $allParams['session']['CDAUTHCRMBONUS'];
        					$codLojaCrm = $allParams['session']['CDCODECRMBONUS'];

        					$idsBonus = $allParams['DATASALE']['fidelityDetails'][0];
        					$idsBonus = $idsBonus['data']['ids_bonus'];

    						$headers = array(
    							'content-Type: application/json',
    							'authorization: '. base64_encode($autorizacaoCrm),
    							'ec: ' . base64_encode($codLojaCrm)
    						);

    						$body = array(
    							'name' => 'voce',
    	    					'phone' => $telefone,
    	   						'amount' => $amount,
    	   						'type' => 'geracao',
    	   						'ids_bonus' => $idsBonus,
    	   						'bonus_utilizado' => $allParams['DATASALE']['FIDELITYDISCOUNT'],
    	   						'loja_b2b_id' => '',
    	   						'amount_b2b' => '',
    	   						'transaction_id' => $allParams['TIPORECE'][0]["TRANSACTIONID"],
    	   						'transaction_code' => $allParams['TIPORECE'][0]["TRANSACTIONCODE"]
    						);

    						$body = json_encode($body);

	    					$answer = $this->paymentService->callCrmBunus($URL, $headers, $body);

							$URL = $allParams['session']['CDURLCRMBONUS'] . '/api/pos/add-total-sales';
	    					$body = array(
	    						'phone' => $telefone,
    							'name' => 'voce',
    	   						'amount' => $amount,
    	   						'bonusId' => $answer['data']['bonusId'],
    	   						'orderId' => $answer['data']['orderId']
    						);

    						$body = json_encode($body);

	    					$answer = $this->paymentService->callCrmBunus($URL, $headers, $body);

	    				} catch (\Exception $e){
    						Exception::logException($e);
    						$result['message'] = $e->getMessage();
							$result['resetSaleCode'] = true;
						}
					}
				} else {
					$result['resetSaleCode'] = true;
				}
				$this->paymentService->updateSaleCode(
					$result['error'] ? 'E' : 'C', // E - erro, C - concluído com sucesso
					json_encode($result),
					$allParams['session']['CDFILIAL'],
					$allParams['NRVENDAREST'],
					$allParams['NRCOMANDA'],
					$allParams['saleCode']
				);

                // Apaga o XML de retorno.
                if ($result['error'] && $result['message'] == "UpdateSaleException"){
                    $result['message'] = 'Ocorreu um erro na hora de gravar a nota no banco. Por favor, tente novamente.';
                    $result['resetSaleCode'] = false;
                }
                else {
                    try {
                        unlink($storedXMLPath . $allParams['saleCode'] . ".xml");
                    } catch (\Exception $e){
                        // Não fazer nada.
                    }
                }
            }
		} catch (\Exception $e) {
			Exception::logException($e);
			$result['message'] = $e->getMessage();
			$result['resetSaleCode'] = true;

			try {
				$this->paymentService->updateSaleCode(
					'E', // E - erro
					json_encode($result),
					$allParams['session']['CDFILIAL'],
					$allParams['NRVENDAREST'],
					$allParams['NRCOMANDA'],
					$allParams['saleCode']
				);
			} catch (\Exception $e) {
				// do nothing
			}
		}

		return $result;
	}

	public function advancePayAccount($allParams){

        set_time_limit(0); // bloqueia timeout na venda

        $result = array(
            'error'   => true,
            'message' => ''
        );

        try {
            // Verifica se a nota para esta venda já foi transmitida.
            try {
                if ($allParams['session']['IDTPEMISSAOFOS'] == 'SAT'){
                    $year = date('Y');
                    $month = date('m');
                    $day = date('d');

                    $XMLPath = realpath('../../../') . DIRECTORY_SEPARATOR . 'SAT' . DIRECTORY_SEPARATOR . $allParams['session']['CDFILIAL'] . DIRECTORY_SEPARATOR . $allParams['session']['CDCAIXA'] . DIRECTORY_SEPARATOR . $year . '-' . $month . '-' . $day . DIRECTORY_SEPARATOR . 'LOGS' . DIRECTORY_SEPARATOR;
                    $vendaTransmitida = file_exists($XMLPath . $allParams['saleCode'] . ".xml");
                    $storedXMLPath = $XMLPath;
                } else if($allParams['session']['IDTPEMISSAOFOS'] == 'FNC'){
                    $nfceData = $this->caixaAPI->getNFCEData($allParams['session']['CDFILIAL']);

                    $DTEMISSAO = new \DateTime();
                    $typePath = $nfceData['IDAMBTRABNFCE'] == '2' ? 'homologacao' : 'producao';
                    $XMLPath = realpath('../../../') . DIRECTORY_SEPARATOR . 'NFCE' . DIRECTORY_SEPARATOR   . 'NOTAS' . DIRECTORY_SEPARATOR . $typePath . DIRECTORY_SEPARATOR;
                    $nfceMoment = $allParams['session']['NRORG'] . '-' . $allParams['session']['CDFILIAL'] . '-' . $allParams['session']['CDCAIXA'] . '_' . $DTEMISSAO->format('Y-m-d') . DIRECTORY_SEPARATOR;
                    $storedXMLPath = $XMLPath . "ENVIADAS" . DIRECTORY_SEPARATOR . $nfceMoment;

                    $vendaTransmitida = file_exists($storedXMLPath . $allParams['saleCode'] . ".xml");
                }
            } catch (\Exception $e){
                $vendaTransmitida = false;
            }

            // alteração realizada a pedido do madero
			// altera o cdtiporece de acordo com a carteira digital selecionada na integração da sitef
			if ($allParams['TIPORECE'][0]['IDTIPORECE'] === 'H' && isset($allParams['TIPORECE'][0]['digitalWallets'])) {
				$newTiporeceName = self::getDigitalWalletName($allParams['TIPORECE'][0]['digitalWallets'], $allParams['TIPORECE'][0]['selectedDigitalWallet']);

				$recebimentosH = $this->paymentService->getTiporeceH();

				$updatedTiporece = self::checkIfTiporeceIsSaved($newTiporeceName, $recebimentosH);

				if ($updatedTiporece == null) {
					$result = Array(
						'error' => true,
						'message' => 'Tipo de recebimento selecionado "' . $newTiporeceName . '" não foi encontrado.'
					);

					return $result;
				}

				$tiporeceBackupValues = array(
					'CDTIPORECE' => $allParams['TIPORECE'][0]['CDTIPORECE'],
					'NMTIPORECE' => $allParams['TIPORECE'][0]['DSBUTTON']
				);

				$allParams['TIPORECE'][0]['CDTIPORECE'] = $updatedTiporece['CDTIPORECE'];
				$allParams['TIPORECE'][0]['DSBUTTON'] = $updatedTiporece['NMTIPORECE'];
			}

			$estadoCaixa = $this->caixaAPI->getEstadoCaixa($allParams['session']['CDFILIAL'], $allParams['session']['CDCAIXA'], $allParams['session']['NRORG']);
	        if($estadoCaixa['estado'] === 'aberto' || $allParams['session']['IDCOLETOR'] === 'C') {
				/*** IMPEDE VENDAS DUPLICADAS ***/
				// Verifica se a venda já foi realizada
				// IDSTORDER = E - erro, C - concluído com sucesso, P - pendente (em processamento)
				$saleCodeObject = $this->paymentService->checkSaleCode($allParams['session']['CDFILIAL'], $allParams['NRVENDAREST'], $allParams['NRCOMANDA'], $allParams['saleCode']);
				if ($saleCodeObject && !$vendaTransmitida) {
					if ($saleCodeObject['IDSTORDER'] == 'P') {
						$result['error'] = true;

						$reqDate = substr($saleCodeObject['DTHRINCREQ'], 0, -4);
						$reqDate = date_create($reqDate);
						$minutesDiff = $reqDate->diff(new \DateTime())->format('%I');
						$secondsDiff = $reqDate->diff(new \DateTime())->format('%S');

						if(intval($minutesDiff) >= 1) {
							$this->paymentService->updateSaleCode(
								'E', // E - erro
								json_encode($result),
								$allParams['session']['CDFILIAL'],
								$allParams['NRVENDAREST'],
								$allParams['NRCOMANDA'],
								$allParams['saleCode']
							);

							$result['message'] = 'Tempo para processamento da venda excedido. Favor tentar novamente.';
							$result['resetSaleCode'] = true;
						} else {
							$result['message'] = 'A requisição está em processamento, por favor aguarde um momento e tente novamente. <br><br>
								Tempo decorrido: ' . $minutesDiff . ':' . $secondsDiff . '<br> Tempo máximo: 01:00';
						}
					} else {
						$result = json_decode($saleCodeObject['DSOPERACAO'], true);
						if ($saleCodeObject['IDSTORDER'] == 'E') {
							// última venda com erro; reseta o code (CDORDERWAITER) para uma nova venda
							$result['resetSaleCode'] = true;
						}
					}
				} else {
                    if (!$vendaTransmitida){ // Caso a venda já tiver sido transmitida, não grava o código.
                        $this->paymentService->insertSaleCode(
                            'P', // P - pendente (em processamento)
                            json_encode($result),
                            $allParams['session']['CDFILIAL'],
                            $allParams['NRVENDAREST'],
                            $allParams['NRCOMANDA'],
                            $allParams['saleCode']
                        );
                    }

                    $VRDESCONTO = $allParams['DATASALE']['VRDESCONTO'];

					$CDFILIAL = $allParams['session']['CDFILIAL'];
					$CDLOJA   = $allParams['session']['CDLOJA'];
					$CDCAIXA  = $allParams['session']['CDCAIXA'];
					$NRORG    = $allParams['session']['NRORG'];
					$CDOPERADOR = $allParams['session']['CDOPERADOR'];

					$allParams['CDCLIENTE']    = $allParams['CDCLIENTE'] ? $allParams['CDCLIENTE'] : null;
					$allParams['CDCONSUMIDOR'] = $allParams['CDCONSUMIDOR'] ? $allParams['CDCONSUMIDOR'] : null;
					if (!$allParams['CDCONSUMIDOR']){
						$NMCONSVEND = 'CONSUMIDOR FINAL';
					} else {
						$NMCONSVEND =  $this->paymentService->findNMCONSUMIDOR($allParams['CDCLIENTE'], $allParams['CDCONSUMIDOR'], $NRORG);
						if (!$allParams['NRINSCRCONS']) {
							$allParams['NRINSCRCONS'] = $this->paymentService->findNRCPFRESPCON($allParams['CDCLIENTE'], $allParams['CDCONSUMIDOR'], $NRORG);
						}
					}
					if ($allParams['NOMECONS'] != null && $allParams['NOMECONS'] != ""){
						$NMCONSVEND = $allParams['NOMECONS'];
					}
					$simulatePrinter = false;
					$simulateSaleValidation = false;

					$result = $this->paymentService->initiatesAnticipate(
						$allParams['chave'],
						$allParams['DATASALE'],
						$allParams['TIPORECE'],
						$allParams['CDCLIENTE'],
						$allParams['CDCONSUMIDOR'],
						$allParams['NRMESA'],
						$allParams['NRVENDAREST'],
						$allParams['NRCOMANDA'],
						$allParams['NRINSCRCONS'],
						$allParams['arrayPosicoes']
					);

					//altera campo ADIANTAMENTO da COMANDAVEN para S
			    	$this->paymentService->updateAdiantamentoCOMANDAVEN($allParams['chave'], $CDFILIAL, $allParams['NRCOMANDA'], $allParams['NRVENDAREST']);

					//pega campo ADIANTAMENTO para validar no front
					$infoComanda = $this->paymentService->getAdvanceInfoCOMANDAVEN(
						$allParams['chave'],
						$CDFILIAL,
						$allParams['NRVENDAREST'],
						$allParams['NRCOMANDA']
					);
					$result['IDADIANTAMENTO'] = $infoComanda;

					if (!$result['error']) {
						$resultDadosMesa = $this->tableService->dadosMesa(
							$CDFILIAL,
							$CDLOJA,
							$allParams['NRCOMANDA'],
							$allParams['NRVENDAREST'],
							false
						);
						$result['IDSTMESAAUX'] = !empty($resultDadosMesa['IDSTMESAAUX']) ? $resultDadosMesa['IDSTMESAAUX'] : 'D';

						if ($allParams['session']['IDCRMBONUS'] == 'S' && $allParams['DATASALE']['PHONECRM'] != NULL) {

							try{
								$telefone = $allParams['DATASALE']['PHONECRM'];
	        					$amount = $allParams['DATASALE']['TOTAL'];
	        					$URL = $allParams['session']['CDURLCRMBONUS'] . '/api/pos/finaliza-venda-full';
	        					$autorizacaoCrm = $allParams['session']['CDAUTHCRMBONUS'];
	        					$codLojaCrm = $allParams['session']['CDCODECRMBONUS'];

	        					$idsBonus = $allParams['DATASALE']['fidelityDetails'][0];
	        					$idsBonus = $idsBonus['data']['ids_bonus'];

        						$headers = array(
        							'content-Type: application/json',
        							'authorization: '. base64_encode($autorizacaoCrm),
        							'ec: ' . base64_encode($codLojaCrm)
        						);

        						$body = array(
        							'name' => 'voce',
	    	    					'phone' => $telefone,
	    	   						'amount' => $amount,
	    	   						'type' => 'geracao',
	    	   						'ids_bonus' => $idsBonus,
	    	   						'bonus_utilizado' => $allParams['DATASALE']['FIDELITYDISCOUNT'],
	    	   						'loja_b2b_id' => '',
	    	   						'amount_b2b' => '',
	    	   						'transaction_id' => $allParams['TIPORECE'][0]["TRANSACTIONID"],
	    	   						'transaction_code' => $allParams['TIPORECE'][0]["TRANSACTIONCODE"]
	    						);

	    						$body = json_encode($body);
		    					$answer = $this->paymentService->callCrmBunus($URL, $headers, $body);

								$URL = $allParams['session']['CDURLCRMBONUS'] . '/api/pos/add-total-sales';
		    					$body = array(
		    						'phone' => $telefone,
        							'name' => 'voce',
	    	   						'amount' => $amount,
	    	   						'bonusId' => $answer['data']['bonusId'],
	    	   						'orderId' => $answer['data']['orderId']
	    						);

	    						$body = json_encode($body);

		    					$answer = $this->paymentService->callCrmBunus($URL, $headers, $body);

		    					$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CallCrmBunus', $answer));

		    				} catch (\Exception $e){
        						Exception::logException($e);
        						$response->addMessage(new Message($e->getMessage()));
    						}
						}
					} else {
						$result['resetSaleCode'] = true;
					}
					$this->paymentService->updateSaleCode(
						$result['error'] ? 'E' : 'C', // E - erro, C - concluído com sucesso
						json_encode($result),
						$allParams['session']['CDFILIAL'],
						$allParams['NRVENDAREST'],
						$allParams['NRCOMANDA'],
						$allParams['saleCode']
					);

                    // Apaga o XML de retorno.
                    if ($result['error'] && $result['message'] == "UpdateSaleException"){
                        $result['message'] = 'Ocorreu um erro na hora de gravar a nota no banco. Por favor, tente novamente.';
                        $result['resetSaleCode'] = false;
                    }
                    else {
                        try {
                            unlink($storedXMLPath . $allParams['saleCode'] . ".xml");
                        } catch (\Exception $e){
                            // Não fazer nada.
                        }
                    }

				}
		 	} else {
	        	$result['message'] = 'Não foi possível realizar a venda. O caixa se encontra fechado.';
	        }
		} catch (\Exception $e) {
			Exception::logException($e);
			$result['message'] = $e->getMessage();
			$result['resetSaleCode'] = true;

			try {
				$this->paymentService->updateSaleCode(
					'E', // E - erro
					json_encode($result),
					$allParams['session']['CDFILIAL'],
					$allParams['NRVENDAREST'],
					$allParams['NRCOMANDA'],
					$allParams['saleCode']
				);
			} catch (\Exception $e) {
				// do nothing
			}
		}

		return $result;
	}

    public function validaTipoConsReceb(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->util->getParams($params);

        $TIPORECE      = $params['TIPORECE'];
        $CDCLIENTE     = $params['CDCLIENTE'];
        $CDCONSUMIDOR  = $params['CDCONSUMIDOR'];

        $result = $this->paymentService->validaTipoConsReceb($TIPORECE, $CDCLIENTE, $CDCONSUMIDOR);

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('validaTipoConsReceb', array('boTipoConsReceb'=>$result)));
	}

    public function validaVendaMesa(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
		$params = $this->util->getParams($params);

        $chave   = !empty($params['chave']) ? $params['chave'] : null;
        $session = $this->util->getSessionVars($chave);
        if (isset($params['IDMODULO'])){
            $session['IDMODULO'] = $params['IDMODULO'];
		}
		if($session['IDMODULO']!='M'){
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('validaVendaMesa', array('error'=>false,'message'=>'')));
			return;
		}
        $CDFILIAL    = $session['CDFILIAL'];
        $CDLOJA      = $session['CDLOJA'];
		$CDCLIENTE   = $params['CDCLIENTE'];
		$NRVENDAREST = $params['NRVENDAREST'];
		$NRCOMANDA   = $params['NRCOMANDA'];
		$NRMESA  	 = $params['NRMESA'];
		$DELIVERY  	 = $params['DELIVERY'];
		$IDMODULO  	 = $session['IDMODULO'];
		$DATASALE  	 = $params['DATASALE'];
		$NRORG  	 = $session['NRORG'];

		$arrayPosicoes = self::formatPositions($params['NRLUGARMESA']);

		if ($DELIVERY){
			$dadosMesa = $this->deliveryService->getInfoOrder($CDFILIAL, $NRVENDAREST);
		} else {
			$dadosMesa = $this->tableService->getTablesButNotGrouped(
				$CDFILIAL, $NRVENDAREST, $NRCOMANDA,
				$NRMESA, $NRORG, $IDMODULO
			);
		}

		$VRDESCONTO = $DATASALE['VRDESCONTO'] + $DATASALE['FIDELITYVALUE'];
		$VRDESCCASHUDS = isset($DATASALE['VRDESCCASHUDS'])?$DATASALE['VRDESCCASHUDS']:0;
		$VRTXSEVENDA = $DATASALE['VRTXSEVENDA'];
		$VRACRCOMANDA = $this->deliveryService->getTaxaEntrega($CDFILIAL, $NRVENDAREST);
		$TOTALPRODUTOS = $DATASALE['TOTAL'] + $DATASALE['VRTXSEVENDA'] + $DATASALE['VRCOUVERT'];
		$REPIQUE = isset($DATASALE['REPIQUE'])? $DATASALE['REPIQUE'] : null;

		$result = $this->vendaAPI->validaVendaComanda(
			$CDFILIAL, $dadosMesa, $arrayPosicoes,
			$CDLOJA, $VRDESCONTO, $VRTXSEVENDA, $CDCLIENTE,
			$DELIVERY, $VRACRCOMANDA, $TOTALPRODUTOS, $REPIQUE,
			$VRDESCCASHUDS
		);

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('validaVendaMesa', $result));
    }

    private function prepareDeliveryDataSale(&$params){
        if($params['IDSTCOMANDA'] == 'P'){
            throw new \Exception("Operação não permitida. Este cupom já foi impresso.", 1);
        }
        $params['ITEMVENDA'] = [];
        $paramsDlv = $this->paymentService->getPaymentDelivery($params['CDFILIAL'], $params['NRVENDAREST'], $params['NRCOMANDA']);
        if(isset($paramsDlv['NRLUGARMESA'])){
            $paramsDlv['NRLUGARMESA'] = array($paramsDlv['NRLUGARMESA']);
        }
        $params = $paramsDlv + $params;
    }

	private function buildObsItemVenda($group, array $obs, $obsDigita) {
		$obsNames = $this->paymentService->getOcorTxts($group, $obs);

		if (!empty($obsDigita)) {
			$obsNames[] = $obsDigita;
		}

		return implode('; ', $obsNames);
	}

	private function handleProducts($session, $ITEMVENDA, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $CDVENDEDOR) {
		$handledProducts = array();
		$ordemImp = 1;
        $voucherUsage = array();
		foreach ($ITEMVENDA as $ITEM){
			$newProduct = array();
			$ITEM = self::fillEmptyProperties($ITEM);
			$newProduct['NMPRODUTO'] = $ITEM['DSBUTTON'];
			$newProduct['NMPRODPROMOCAO'] = $ITEM['DSBUTTON'];
			$newProduct['CDPRODUTO'] = $ITEM['CDPRODUTO'];
			$newProduct['QTPRODVEND'] = $ITEM['QTPRODCOMVEN'];
			$newProduct['QTPRODCOMVEN'] = $ITEM['QTPRODCOMVEN'];
			$newProduct['VRUNITVEND'] = $ITEM['PRITEM'];
            $newProduct['VRUNITVENDCL'] = $ITEM['VRPRECITEMCL'];
            $newProduct['REALSUBSIDY'] = $ITEM['REALSUBSIDY'];
			$newProduct['NRLUGARMESA'] = '1';
			$newProduct['VRDESITVEND'] = ($ITEM['VRDESITVEND'] > $ITEM['PRITEM']) ? $ITEM['PRITEM'] : $ITEM['VRDESITVEND'];
			$newProduct['VRACRITVEND'] = $ITEM['VRACRITVEND'];
			$newProduct['IDSITUITEM'] = 'A';
			$newProduct['IDTIPOITEM'] = null;
			$newProduct['ORDEMIMP'] = $ordemImp;
			$newProduct['IDTIPOCOMPPROD'] = $ITEM['IDTIPOCOMPPROD'];
			$newProduct['IDIMPPROMOCAO'] = $ITEM['IDTIPOCOMPPROD'] == '3' ? 'S' : 'N';
			$newProduct['IDIMPPRODUTO'] = $ITEM['IDIMPPRODUTO'];
			$newProduct['TXPRODCOMVEN'] = !empty($ITEM['TXPRODCOMVEN']) ? $ITEM['TXPRODCOMVEN'] : null;
			$newProduct['CDSUPERVISOR'] = null;
			$newProduct['DSOBSITEMVENDA'] = $this->buildObsItemVenda($session['CDGRPOCORPED'], $ITEM['CDOCORR'], $ITEM['DSOCORR_CUSTOM']);
			$newProduct['DSOBSPEDDIGITA'] = $ITEM['DSOCORR_CUSTOM'];
			$newProduct['DTHRPRODCANVEN'] = $ITEM['DTHRPRODCANVEN'];
			$newProduct['OBSERVACOES'] = array_map(function($obs) use ($session) {
				return array(
					'CDOCORR' => $obs,
					'CDGRPOCOR' => $session['CDGRPOCORPED']
				);
			}, $ITEM['CDOCORR']);
			$newProduct['CDGRPOCOR'] = null;
            $newProduct['ATRASOPROD'] = $ITEM['ATRASOPROD'] == 'Y' ? $session['NRATRAPADRAO'] : null;
            $newProduct['IDORIGEMVENDA'] = ($ITEM['TOGO'] == 'Y' && $session['IDCTRLPEDVIAGEM'] == 'S') ? 'TGO_BAL' : null;
			$newProduct['DSOBSDESCIT'] = null;
			$newProduct['CDGRPOCORDESCIT'] = null;
			$newProduct['CDVENDEDOR'] = $CDVENDEDOR;
            $newProduct['CDPRODPROMOCAO'] = empty($ITEM['PRODUTOS']) ? null : $ITEM['CDPRODUTO'];
			$ordemImpPromo = 0.01;
			$comboItems = array();
			foreach ($ITEM['PRODUTOS'] as $currentComboItem) {
				$newItem = array();
                $newItem['CDPRODPROMOCAO'] = $ITEM['CDPRODUTO'];
				$newItem['CDPRODUTO'] = $currentComboItem['CDPRODUTO'];
				$newItem['NMPRODUTO'] = $currentComboItem['DSBUTTON'];
				$newItem['QTPRODVEND'] = $currentComboItem['QTPRODCOMVEN'] * $ITEM['QTPRODCOMVEN'];
				$newItem['QTPRODCOMVEN'] = $currentComboItem['QTPRODCOMVEN'] * $ITEM['QTPRODCOMVEN'];
				$newItem['VRUNITVEND'] = $currentComboItem['PRECO'];
                $newItem['VRUNITVENDCL'] = $currentComboItem['VRPRECITEMCL'];
                $newItem['REALSUBSIDY'] = $currentComboItem['REALSUBSIDY'];
				$newItem['VRDESITVEND'] = ($currentComboItem['VRDESITVEND'] > $currentComboItem['PRECO']) ? $currentComboItem['PRECO'] : $currentComboItem['VRDESITVEND'];
				$newItem['NRLUGARMESA'] = '1';
				$newItem['ATRASOPROD'] = $currentComboItem['ATRASOPROD'] == 'Y' ? $session['NRATRAPADRAO'] : null;
                $newItem['IDORIGEMVENDA'] = ($currentComboItem['TOGO'] == 'Y' && $session['IDCTRLPEDVIAGEM'] == 'S') ? 'TGO_BAL' : null;
				$newItem['ORDEMIMP'] = $ordemImp + $ordemImpPromo;
				$newItem['IDTIPOCOMPPROD'] = '1';
				$newItem['IDIMPPROMOCAO'] = 'X';
				$newItem['VRACRITVEND'] = $currentComboItem['VRACRITVEND'];
				$newItem['IDSITUITEM'] = 'A';
				$newItem['IDTIPOITEM'] = null;
				$newItem['TXPRODCOMVEN'] = !empty($currentComboItem['TXPRODCOMVEN']) ? $currentComboItem['TXPRODCOMVEN'] : null;
				$newItem['OBSERVACOES'] = array_map(function($obs) use ($session) {
					return array(
						'CDOCORR' => $obs,
						'CDGRPOCOR' => $session['CDGRPOCORPED']
					);
				}, $currentComboItem['CDOCORR']);
				$newItem['IDAPLICADESCPR'] = $currentComboItem['IDAPLICADESCPR'];
				$newItem['IDPERVALORDES'] = $currentComboItem['IDPERVALORDES'];
				$newItem['CDGRPOCOR'] = null;
				$newItem['DSOBSDESCIT'] = null;
				$newItem['CDGRPOCORDESCIT'] = null;
				$newItem['CDVENDEDOR'] = $CDVENDEDOR;
				$newItem['VOUCHER'] = $this->checkVoucherUsage($currentComboItem, $voucherUsage);
				if($ITEM['IDIMPPRODUTO'] == '2'){
					$newItem['IDIMPPRGRPROMO'] = 'N';
				} else {
					$newItem['IDIMPPRGRPROMO'] = $currentComboItem['IDIMPGRPROMO'];
				}

                $newItem['IDOBSERVACAOAD'] = 'N';

                $productDetails = $this->vendaAPI->getProductDetails($newItem['CDPRODUTO']);
                $newItem['IDPESAPROD'] = $productDetails['IDPESAPROD'];
                $newItem['IDIMPRODUVEZ'] = $productDetails['IDIMPRODUVEZ'];

                $newItem['CDCAMPCOMPGANHE'] = null;
                $newItem['DTINIVGCAMPCG'] = null;

				array_push($comboItems, $newItem);

				$ordemImpPromo += 0.01;
			}

            $newProduct['VOUCHER'] = $this->checkVoucherUsage($ITEM, $voucherUsage);
            if (!array_key_exists('IMPRIMESIM', $ITEM)) $ITEM['IMPRIMESIM'] = false; // Correção temporária.
			$newProduct['IDIMPPRGRPROMO'] = !!$ITEM['IMPRIMESIM'] ? 'S' : 'N';
            $newProduct['IDOBSERVACAOAD'] = 'N';

            $productDetails = $this->vendaAPI->getProductDetails($newProduct['CDPRODUTO']);
            $newProduct['IDPESAPROD'] = $productDetails['IDPESAPROD'];
            $newProduct['IDIMPRODUVEZ'] = $productDetails['IDIMPRODUVEZ'];

            if (!empty($ITEM['CAMPANHA'])){
                $newProduct['CDCAMPCOMPGANHE'] = $ITEM['CAMPANHA'];
                if ($this->util->databaseIsOracle()){
                    $newProduct['DTINIVGCAMPCG'] = \DateTime::createFromFormat('Y-m-d H:i:s', $ITEM['DTINIVGCAMPCG']);
                }
                else {
                    $newProduct['DTINIVGCAMPCG'] = \DateTime::createFromFormat('Y-m-d H:i:s.u', $ITEM['DTINIVGCAMPCG']);
                }
            }
            else {
                $newProduct['CDCAMPCOMPGANHE'] = null;
                $newProduct['DTINIVGCAMPCG'] = null;
            }

			if (!empty($comboItems)){
				// rotina específica do Madero
				$this->util->validaDescontoDiferenciado($CDFILIAL, $ITEM['CDPRODUTO'], $comboItems, 'VRDESITVEND');
			}

			$newProduct['itensCombo'] = $comboItems;

			array_push($handledProducts, $newProduct);

			$ordemImp++;
		}

		return $handledProducts;
	}

    private function checkVoucherUsage($product, $voucherUsage){
        if (empty($product['VOUCHER'])) return null;
        else {
            $voucher = $this->paymentService->validateVoucher($product['CDPRODUTO'], null, null, $product['VOUCHER']['CDCUPOMDESCFOS']);
            // Verifica se o voucher já foi utilizado neste pedido.
            if (in_array($product['VOUCHER']['CDCUPOMDESCFOS'], $voucherUsage)){
                throw new \Exception("O voucher " . $product['VOUCHER']['CDCUPOMDESCFOS'] . " é de uso único, mas está sendo aplicado em mais de um produto. Certifique-se de que apenas um produto esteja com este voucher aplicado.");
            }
            else {
                array_push($voucherUsage, $product['VOUCHER']['CDCUPOMDESCFOS']);
            }
            return $voucher;
        }
    }

	private function fillEmptyProperties($item) {

		if (!isset($item['CDOCORR'])) {
			$item['CDOCORR'] = null;
		}
		if (!isset($item['DSOCORR_CUSTOM'])) {
			$item['DSOCORR_CUSTOM'] = null;
		}
		if (!isset($item['PRODUTOS'])) {
			$item['PRODUTOS'] = array();
		}
		if(!isset($item['DTHRPRODCANVEN'])){
			$item['DTHRPRODCANVEN'] = null;
		}
		return $item;
	}

	public function getTransactioncode(Request\Filter $request, Response $response) {
		try {
			$session  = $this->util->getSessionVars(null);
			$CDFILIAL = $session['CDFILIAL'];
			$CDCAIXA  = $session['CDCAIXA'];
			$dateTime = date_format(new \DateTime('NOW', new \DateTimeZone('America/Sao_Paulo')), 'd/m/Y');

			$this->util->newCode('SEQTEF'.$CDFILIAL.$CDCAIXA.$dateTime);
			$SEQTEF = $this->util->getNewCode('SEQTEF'.$CDFILIAL.$CDCAIXA.$dateTime, 10);
			$result = array('SEQTEF' => $SEQTEF);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getTransactioncode', $result));
		} catch (\Exception $e) {
			Exception::logException($e);
			$response->addMessage(new Message($e->getMessage()));
		}
	}

	public function getTipoProd(Request\Filter $request, Response $response){
		$params = $request->getFilterCriteria()->getConditions();
		$params = $this->util->getParams($params);
		$ITEMVENDA = $params['ITEMVENDA'];
		$CDCLIENTE = $params['CDCLIENTE'];
		$CDCONSUMIDOR = $params['CDCONSUMIDOR'];

		$session = $this->util->getSessionVars(null);
		$CDFILIAL = $session['CDFILIAL'];
		$NRORG = $session['NRORG'];

		$result = $this->consumerService->getTotProdBasicoExtra($ITEMVENDA, $CDFILIAL, $NRORG, $CDCLIENTE, $CDCONSUMIDOR);

		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetTipoProd', $result));
		
	}

	public function getConsumerLimit(Request\Filter $request, Response $response) {
		try {
			$params = $request->getFilterCriteria()->getConditions();
			$params = $this->util->getParams($params);
			$CDCLIENTE = $params['CDCLIENTE'];
			$CDCONSUMIDOR  = $params['CDCONSUMIDOR'];
			$type  = $params['type'];

            $session = $this->util->getSessionVars(null);

			$CDFILIAL = $session['CDFILIAL'];
            $CDLOJA = $session['CDLOJA'];
            $CDOPERADOR = $session['CDOPERADOR'];
            $IDPERCOMVENCPDC = $session['IDPERCOMVENCPDC'];
            $IDSALDEXTDEBCON = $session['IDSALDEXTDEBCON'];
            $NRDIAFECHAFOLHA = $session['NRDIAFECHAFOLHA'];

			if($type == 'debito'){
                $this->consumerService->atualizaSaldoMoviClie($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDOPERADOR);
				$clientLimit = $this->consumerService->getLimitDebConsumer($CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $IDSALDEXTDEBCON, $NRDIAFECHAFOLHA);
			}
            else if($type == 'credito'){
				//ainda nao tratando mais de uma familia aqui
                $clientLimit = $this->consumerService->getLimiteCredDisponivel($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);
			}
            else if($type == 'all') {
                $this->consumerService->atualizaSaldoMoviClie($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDOPERADOR);
                $clientLimit = array(
					'debito' => $this->consumerService->getLimitDebConsumer($CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $IDSALDEXTDEBCON, $NRDIAFECHAFOLHA),
					'credito' => $this->consumerService->getLimiteCredDisponivel($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR)
				);
			}

            $params = array(
                'CDCLIENTE' => $CDCLIENTE,
                'CDCONSUMIDOR' => $CDCONSUMIDOR
            );
            $consumerDetails = $this->consumerService->populaDadosConsumidor($params);
            $clientLimit['IDTPVENDACONS'] = $consumerDetails['IDTPVENDACONS'];
            $clientLimit['IDPERCOMVENCPDC'] = $IDPERCOMVENCPDC === 'S';
            $clientLimit['IDSALDEXTDEBCON'] = $IDSALDEXTDEBCON;

			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetConsumerLimit', $clientLimit));
		} catch (\Exception $e) {
			Exception::logException($e);
			$response->addMessage(new Message(utf8_encode($e->getMessage())));
		}
	}

	public function formatPositions($positions){
		return array_map(function($position){
			return str_pad($position, 2, '0', STR_PAD_LEFT);
		}, $positions);
	}

	public function printTEFVoucher(Request\Filter $request, Response $response){
		try {
			$params = $request->getFilterCriteria()->getConditions();
			$params = $this->util->getParams($params);
			$session = $this->util->getSessionVars($params['chave']);

			$printTEFVoucherResult = $this->paymentService->printTEFVoucher($params['arrTEFVoucher'], $session['CDFILIAL'], $session['CDCAIXA'], $session['NRORG']);
			if ($printTEFVoucherResult['error']){
				$response->addMessage(new Message($printTEFVoucherResult['message']));
			} else {
				$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('PrintTEFVoucher', $printTEFVoucherResult));
			}
		} catch (\Exception $e) {
			Exception::logException($e);
			$response->addMessage(new Message('Ocorreu um problema na impressão do comprovante TEF. ' . $e->getMessage()));
		}
	}

    public function processQRCodeSale(Request\Filter $request, Response $response){
        $result = array(
            'error'   => true,
            'message' => '',
            'resetSaleCode' => true
        );

        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);
            $session = $this->util->getSessionVars($chave);
            $saleInfo = self::formatSaleCode($params['QRCODE']);

            $validaVenda = $this->paymentService->buscaVendaRealizada($saleInfo['FIL'], $saleInfo['NVR'], $saleInfo['NCD']);
            if (!empty($validaVenda)){
                throw new \Exception("Venda já processada.");
            }

            $consumer = $this->consumerService->populaDadosConsumidor(array('CDCLIENTE' => $saleInfo['CLIE'], 'CDCONSUMIDOR' => $saleInfo['CONS']));
            if (!empty($consumer) && empty($params['CPF'])){
                if (!empty($consumer['NRCPFRESPCON'])) $params['CPF'] = $consumer['NRCPFRESPCON'];
            }

            $saleInfo['chave'] = $params['chave'];
            $saleInfo['saleCode'] = $params['saleCode'] . $params['chave'];

            $saleInfo['TIPORECE'] = self::formatRec($saleInfo);
            $saleInfo['ITEMVENDA'] = self::formatProds($saleInfo['PRODS']);
            $saleInfo['DATASALE'] = self::prepareDatasale($saleInfo['TIPORECE'][0]['VRMOVIVEND']);

            $TIPORECE      = $saleInfo['TIPORECE'];
            $ITEMVENDA     = $saleInfo['ITEMVENDA'];
            $DATASALE      = $saleInfo['DATASALE'];
            $CDCLIENTE     = $saleInfo['CLIE'];
            $CDCONSUMIDOR  = $saleInfo['CONS'];
            $NRMESA        = "";
            $NRPESMESAVEN  = "";
            $NRVENDAREST   = $saleInfo['NVR'];
            $NRCOMANDA     = $saleInfo['NCD'];
            $chave         = $saleInfo['chave'];
            $saleCode      = $saleInfo['saleCode'];
            $arrayPosicoes = array();
            $TIPODESCONTO  = null;
            $NRINSCRCONS   = !empty($params['CPF']) ? $params['CPF'] : null;
            $EMAIL         = null;
            $NOMECONS      = null;
            $INFORMAEMAIL  = !empty($params['INFORMAEMAIL']) ? $params['INFORMAEMAIL'] : false;
			$DSEMAILCONS   = !empty($params['DSEMAILCONS']) ? $params['DSEMAILCONS'] : null;

            $result = self::payAccount($TIPORECE, $ITEMVENDA, $DATASALE, $CDCLIENTE, $CDCONSUMIDOR, $NRMESA, $NRPESMESAVEN, $NRVENDAREST, $NRCOMANDA, $chave, $saleCode, $arrayPosicoes, $TIPODESCONTO, $NRINSCRCONS, $EMAIL, $NOMECONS, null, null, null, null, null, $session, null, null, null, false, null, $INFORMAEMAIL, $DSEMAILCONS);
        } catch (\Exception $e){
            Exception::logException($e);
            $result['message'] = 'Erro ao processar a venda: ' . $e->getMessage();
        }

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('QRCodeSaleRepository', $result));
    }

    private function formatSaleCode($qrCode){
        try {
            $qrCode = preg_replace('/\*/', '"', $qrCode);
            $qrCode = preg_replace('/\$/', ': ', $qrCode);
            $qrCode = preg_replace('/</', '{', $qrCode);
            $qrCode = preg_replace('/>/', '}', $qrCode);
            $qrCode = preg_replace('/\(/', '[', $qrCode);
            $qrCode = preg_replace('/\)/', ']', $qrCode);
            return json_decode($qrCode, true);
        } catch (\Exception $e){
            throw new \Exception("QR code mal-formatado.");
        }
    }

    private function formatRec($saleInfo){
        try {
            return array(array(
                'CDTIPORECE' => $saleInfo['REC'][0]['CDTREC'],
                'IDTIPORECE' => $saleInfo['REC'][0]['IDTREC'],
                'VRMOVIVEND' => $saleInfo['REC'][0]['VRT'],
                'CDNSUHOSTTEF' => $saleInfo['NSU']
            ));
        } catch (\Exception $e){
            throw new \Exception("QR code mal-formatado.");
        }
    }

    private function formatProds($produtos){
        try {
            $formatedProducts = array();
            $id = 1;
            foreach($produtos as $prod){
                $productInfo = $this->paymentService->getProduct($prod['CDP']);
                if (empty($productInfo)) throw new \Exception('Produto ' . $prod['CDP'] . ' não encontrado.');
                $product = array(
                    "DSBUTTON" => $productInfo['NMPRODUTO'],
                    "CDPRODUTO" => $prod['CDP'],
                    "QTPRODCOMVEN" => $prod['QT'],
                    "PRITEM" => $prod['VR'],
                    "IDTIPOCOMPPROD" => $productInfo['IDTIPOCOMPPROD'],
                    "IDIMPPRODUTO" => $prod['IMPPROD'],
                    "TXPRODCOMVEN" => "",
                    "CDOCORR" => $prod['OBS'],
                    "DSOCORR_CUSTOM" => null,
                    "ATRASOPROD" => "N",
                    "TOGO" => "N",
                    "PRODUTOS" => array() // Não implementado.
                );
                array_push($formatedProducts, $product);
            }
            return $formatedProducts;
        } catch (\Exception $e){
            throw new \Exception("QR code mal-formatado.");
        }
    }

    private function prepareDatasale($paymentValue){
        return array(
              'TOTALVENDA'   => $paymentValue,
              'FALTANTE'     => 0,
              'VALORPAGO'    => $paymentValue,
              'TROCO'        => 0,
              'TOTAL'        => $paymentValue,
              'VRTXSEVENDA'  => 0,
              'VRDESCONTO'   => 0,
              'PCTDESCONTO'  => 0,
              'TIPODESCONTO' => 'P'
        );
    }

    public function savePayment(Request\Filter $request, Response $response) {
		try {
			$params = $request->getFilterCriteria()->getConditions();
        	$params = $this->util->getParams($params);
        	$session = $this->util->getSessionVars(null);

        	$result = $this->paymentService->savePayment($params, $session);

			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('SavePayment', array()));
		} catch (\Exception $e) {
			Exception::logException($e);
			$response->addMessage(new Message($e->getMessage()));
		}
	}

	public function removePayment(Request\Filter $request, Response $response) {
		try {
			$params = $request->getFilterCriteria()->getConditions();
        	$params = $this->util->getParams($params);
        	$session = $this->util->getSessionVars(null);

        	$result = $this->paymentService->handleRemovePayment($params, $session);

			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RemovePayment', array()));
		} catch (\Exception $e) {
			Exception::logException($e);
			$response->addMessage(new Message($e->getMessage()));
		}
	}

	public function getPayments(Request\Filter $request, Response $response) {
		try {
			$params = $request->getFilterCriteria()->getConditions();
        	$params = $this->util->getParams($params);
        	$session = $this->util->getSessionVars(null);

        	$result = $this->paymentService->getPayments($params, $session);

			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetPayments', $result));
		} catch (\Exception $e) {
			Exception::logException($e);
			$response->addMessage(new Message($e->getMessage()));
		}
	}

 	public function trataBandeirasIntegracoes($bandeira, $tipoRecebimento) {
        //Utiliza o retorno de bandeira da Cielo LIO, GetNet, PagSeguro e transforma em código das bandeiras da SITEF.
        $bandeirasIntegracoes = array(
            'VISA'   		     => array ('1' => '00001','2' => '20002'), // Cielo lio / PagSeguro
            'VISA CREDITO'       => array ('00001'), // Getnet
            'VISA ELECTRON'      => array ('20002'), // Getnet
            'MASTERCARD'         => array ('1' => '00002','2' => '20001'), // Cielo lio / Getnet
            'MASTERCARD CREDITO' => array ('00002'),
            'MASTERCARD DEBITO'  => array ('20001'),
            'MASTER'             => array ('1' => '00002','2' => '20001'), // PagSeguro
            'ELO'    		     => array ('1' => '00031','2' => '20032'),
            'ALELO'   	         => array ('F' => '10022','G' => '10021'),
            'SODEXO'   	         => array ('F' => '10029','G' => '10028'),
            'VR'   		         => array ('F' => '10048','G' => '10047'),
            'HIPERCARD'          => array ('20037'), // Cielo lio
            'HIPER'              => array ('1' => '00012', '2' => '20037'), // Getnet
            'SOROCRED'           => array ('00015'),
            'SOROCRED CREDITO'   => array ('00015'), // Getnet
            'SOROCRED DEBITO'    => array ('20071'), // Getnet
            'SOROCRED VOUCHER'   => array ('10037'), // Getnet
            'BEN VISA VALE'      => array ('F' => '10141','G' => '10140'),
            'AMEX'               => array ('00004'),
            'DINERS'             => array ('00003'),
            'SENFF'              => array ('1' => '10155','F' => '10169','G' => '10170'),
            'GOODCARD'           => array ('1' => '00130','2' => '20131','F' => '10134', 'G' => '10133'), // Cielo lio
            'GOOD CARD'          => array ('1' => '00130','2' => '20131','F' => '10134', 'G' => '10133'), // PagSeguro
            'GOOD CARD FUEL'     => array ('10135'), // Getnet
            'TICKET'             => array ('1' => '00072','2' => '20076','F' => '10025', 'G' => '10024'),
            'TICKET RESTAURANTE' => array ('10024'),
            'TICKET ALIMENTACAO' => array ('10025'),
            'AURA'               => array ('00013'),
            'AVISTA'             => array ('00045'),
            'BANESCARD'          => array ('1' => '00035','2' => '20036'),
            'BIQ F'              => array ('10144'),
            'BNB CLUB'           => array ('1' => '00173','2' => '20145'),
            'CABAL'              => array ('1' => '00030','2' => '20003'),
            'CALCARD'            => array ('00090'),
            'CREDSHOP'           => array ('00071'),
            'CRED F'             => array ('10149'),
            'DOTZ'               => array ('00182'),
            'FORT BRASIL'        => array ('00137'),
            'GREEN CARD'         => array ('F' => '10173','G' => '10174'),
            'NUTRICASH'          => array ('10004'),
            'PLANVALE'           => array ('F' => '10072','G' => '10073'),
            'SAPORE BENEFICIOS'  => array ('10009'),
            'SICRED'             => array ('1' => '00041','2' => '20042'),
            'TIP CARD'           => array ('00223'),
            'TRICARD'            => array ('00093'),
            'VALECARD'           => array ('F' => '10237','G' => '10238'),
            'VALE SHOP'          => array ('10158'),
            'VERDE CARD'         => array ('1' => '00086','2' => '20152'),
            'JCB'                => array ('00011'),
            'CREDZ'              => array ('00070'),
            'MAXXVAN'            => array ('1' => '00105','2' => '20106'),
            'CALCARD'            => array ('00090'),
            'BANRICOMPRAS'       => array ('1' => '00084','2' => '20085'),
            'BENESE'             => array ('1' => '00091','2' => '20092','G' => '10192'),
            'SIMCRED'            => array ('00218'),
            'VEROCARD'           => array ('F' => '10176','G' => '10177'),
            'VERDECARD'          => array ('1' => '00086','2' => '20152'), // PagSeguro
            'UP BRASIL'          => array ('F' => '999999','G' => '999998'),
            'ACCORD CARTOES'     => array ('F' => '999997','G' => '999996'),
            'AGIPLAN'            => array ('999995'),
            'AUTO EXPRESSO'      => array ('999994'),
            'BANDES'             => array ('1' => '999993','2' => '999992'),
            'BARIGUI FINANCEIRA' => array ('999991'),
            'BONUS CBA'          => array ('999990'),
            'CENTRAL CARD'       => array ('999989'),
            'CHINA UNION PAY'    => array ('1' => '999988','2' => '999987'),
            'CREDSHOP'           => array ('999986'),
            'CREDSYSTEM'         => array ('999985'),
            'CROMO UP'           => array ('999984'),
            'DACASA FINANCEIRA'  => array ('999983'),
            'E-CARDES'           => array ('999982'),
            'ESPLANADA'          => array ('999981'),
            'FACESP'             => array ('F' => '999980','G' => '999979'),
            'MINAS CARD'         => array ('999978'),
            'OBOE CARD'          => array ('999977'),
            'PLENO CARD'         => array ('F' => '999976','G' => '999975'),
            'REDECOMPRAS'        => array ('999974'),
            'SPOT PROMO'         => array ('999973'),
            'UNIK'               => array ('999972'),
            'VILLELA BENEFICIO'  => array ('999971'),
            'DISCOVER'           => array ('1' => '999970','2' => '999969'),
            'REDE COMPRAS'       => array ('1' => '999968','2' => '999967'),
            'OUROCARD'           => array ('1' => '999966','2' => '999965'),
            'BANRICARD'          => array ('999964'), // PagSeguro
            'GOODVALE'           => array ('999963') // Getnet
        );

		$bandeira = trim($bandeira);
        if (!array_key_exists($bandeira, $bandeirasIntegracoes)) {
            return $bandeira;
        } else if (!array_key_exists($tipoRecebimento, $bandeirasIntegracoes[$bandeira])) {
            if (!array_key_exists(0, $bandeirasIntegracoes[$bandeira])) {
                return $bandeira;
            } else {
                return $bandeirasIntegracoes[$bandeira][0];
            }
        } else {
            return $bandeirasIntegracoes[$bandeira][$tipoRecebimento];
        }
    }

    public function validateVoucher(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);

            $voucher = $this->paymentService->validateVoucher($params['CDPRODUTO'], $params['NRVENDAREST'], $params['NRCOMANDA'], $params['CDCUPOMDESCFOS']);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('VoucherRepository', $voucher));
        } catch (\Exception $e){
            Exception::logException($e);
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function WiBXConvertValue(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);
            $session = $this->util->getSessionVars(null);

            $identifier = $session['CDFILIAL'] . $session['CDCAIXA'] . $params['saleCode'];

            $WiBXValue = $this->WiBXService->quotations($params['value'], $identifier);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('WiBXConvertValue', $WiBXValue));
        } catch (\Exception $e){
            Exception::logException($e);
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function WiBXRequestPayment(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);
            $session = $this->util->getSessionVars(null);

            $identifier = $session['CDFILIAL'] . $session['CDCAIXA'] . $params['saleCode'];

            $paymentDetails = $this->WiBXService->payments($params['value'], $identifier, 'POST', $session['CDFILIAL'], $session['CDCAIXA']);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('WiBXRequestPayment', $paymentDetails));
        } catch (\Exception $e){
            Exception::logException($e);
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function WiBXCancelPayment(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);
            $session = $this->util->getSessionVars(null);

            $identifier = $session['CDFILIAL'] . $session['CDCAIXA'] . $params['saleCode'];

            $cancelResult = $this->WiBXService->payments(null, $identifier, 'DELETE', $session['CDFILIAL'], $session['CDCAIXA']);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('WiBXCancelPayment', $cancelResult));
        } catch (\Exception $e){
            Exception::logException($e);
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function WiBXPaymentStatus(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);
            $session = $this->util->getSessionVars(null);

            $identifier = $session['CDFILIAL'] . $session['CDCAIXA'] . $params['saleCode'];

            $paymentStatus = $this->WiBXService->payments(null, $identifier, 'GET', $session['CDFILIAL'], $session['CDCAIXA']);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('WiBXPaymentStatus', $paymentStatus));
        } catch (\Exception $e){
            Exception::logException($e);
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    private function getDigitalWalletName($digitalWallets, $selectedDigitalWallet) {
    	if (!empty($digitalWallets)) {
    		$digitalWalletName = null;
    		for($idx = 0; $idx < count($digitalWallets); $idx++) {
    			$stringFilterResult = strpos($digitalWallets[$idx], $selectedDigitalWallet);
    			$stringWithoutID = preg_replace('/[^A-Za-z\-]/', '', $digitalWallets[$idx]);
    			if ($stringFilterResult || $stringFilterResult === 0) {
    				$digitalWalletName = strtoupper($stringWithoutID);
    				$isPicpay 		= strpos($digitalWalletName, 'PIC');
    				$isAME 			= strpos($digitalWalletName, 'AME');
    				$isPIX 			= strpos($digitalWalletName, 'PIX');
    				$isMercadoPago  = strpos($digitalWalletName, 'MERCADO');
    				// $isITI 		= strpos($digitalWalletName, 'ITI'); //AINDA NÃO USAM
    				// $isConnectPay 		= strpos($digitalWalletName, 'CONNECT'); //AINDA NÃO USAM
    				if ($isPicpay || $isPicpay === 0) {
    					// $digitalWalletNoSpace = str_replace(' ', '', $digitalWalletName);
    					// $digitalWalletName = substr($digitalWalletNoSpace, 0, -2);
    					$digitalWalletName = 'PICPAY';
    				}
    				else if ($isAME || $isAME === 0) $digitalWalletName = 'AME DIGITAL';
    				else if ($isPIX || $isPIX === 0) $digitalWalletName = 'PIX MERCADOPAGO';
    				else if ($isMercadoPago || $isMercadoPago === 0) $digitalWalletName = 'MERCADO PAGO';
    				// else if ($isITI || $isITI === 0) $digitalWalletName = 'ITI';
    				// else if ($isConnectPay || $isConnectPay === 0) $digitalWalletName = 'CONNECTPAY';
    			}
	    	}
	    	return $digitalWalletName;
    	}
    }

    private function checkIfTiporeceIsSaved($digitalWalletName, $allTiporeceH) {
    	if(!empty($allTiporeceH)) {
		    for($idx = 0; $idx < count($allTiporeceH); $idx++) {
		    	if($allTiporeceH[$idx]['NMTIPORECE'] == $digitalWalletName) {
		    		return $allTiporeceH[$idx];
		    	}
		    }
		}
    }
    public function callCrmBunus(Request\Filter $request, Response $response) {
    	try {
    		$params = $request->getFilterCriteria()->getConditions();
	        $params = $this->util->getParams($params);
	        $session = $this->util->getSessionVars(null);
	        $telefone = $params['telefoneConsumidor'];
	        $amount = $params['TOTALVENDA'];
	        $URL = $session['CDURLCRMBONUS'] . '/api/pos/bonus-disponivel';
	        $autorizacaoCrm = $session['CDAUTHCRMBONUS'];
	        $codLojaCrm = $session['CDCODECRMBONUS'];

        	$headers = array(
        		'content-Type: application/json',
        		'authorization: '. base64_encode($autorizacaoCrm),
        		'ec: ' . base64_encode($codLojaCrm)
        	);

        	$body = array(
	    	    'phone' => $telefone,
	    	   	'amount' => $amount
	    	);

	    	$body = json_encode($body);

		    $answer = $this->paymentService->callCrmBunus($URL, $headers, $body);

	    	$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('CallCrmBunus', $answer));

		} catch (\Exception $e){
        	Exception::logException($e);
        	$response->addMessage(new Message($e->getMessage()));
    	}
    }

    public function getStatusAdiantamento(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->util->getParams($params);

        $chave         = !empty($params['chave']) ? $params['chave'] : null;
		$session 	   = $this->util->getSessionVars($chave);

		$CDFILIAL = $session['CDFILIAL'];
		$NRVENDAREST = $params['NRVENDAREST'];
		$NRCOMANDA = $params['NRCOMANDA'];

		$infoComanda = $this->paymentService->getAdvanceInfoCOMANDAVEN($CDFILIAL, $NRVENDAREST, $NRCOMANDA);

		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetStatusAdiantamento', $infoComanda));
	}

	public function getAdvancedPayment(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->util->getParams($params);

        $chave         = !empty($params['chave']) ? $params['chave'] : null;
		$session 	   = $this->util->getSessionVars($chave);

		$CDFILIAL = $session['CDFILIAL'];
		$CDCAIXA = $session['CDCAIXA'];
		$NRVENDAREST = $params['NRVENDAREST'];
		$NRCOMANDA = $params['NRCOMANDA'];

		$infoAdvancedPayment = $this->paymentService->searchAdvancedPayments($chave, $CDFILIAL, $CDCAIXA, $NRVENDAREST, $NRCOMANDA);

		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetAdvancedPayment', !empty($infoAdvancedPayment) ? $infoAdvancedPayment : array()));
	}

}
